//Very Tiny Tetris
//version 2
//By Sparr
//(c) 2004 Clarence Risher
//sparr0@gmail.com
//Released under the GNU GPL, latest version available at http://www.gnu.org/licenses/gpl.txt

//this enables a binary score display in the status bar
#define BINARYSCORE
//this enables the next piece display
#define NEXTPIECE
//this enables the string "VTTETRIS" or "VTTetris"
#define LABEL
//this enables the author's name
#define SPARR
//this enables F1 for immediate drop
#define IMMDROP

//this define changes from tiny pieces to 4x4 block sprites
//costs about 200 bytes
#define NOTSOTINY
//draws a prettier border around the not-so-tiny board
#define PRETTYBORDER
//draw a 'ghost' piece where the piece would drop
#define GHOSTDROP
//draw the ghost til the last possible second
#define PERSISTENTGHOST

#ifdef NOTSOTINY
//next 4 defines are for big pieces
//do not change LEFTSIDE
//RIGHTSIDE must be <=4
#define LEFTSIDE 0
#define RIGHTSIDE 1
//do not change TOP
//BOTTOM must be <=TOP+20
#define TOP 108
#define BOTTOM 127
#else
//next 4 defines are for tiny pieces
//these are bytes, counting from the left, 0-19.
//RIGHTSIDE must be >= LEFTSIDE and < LCD_LINE_BYTES
#define LEFTSIDE 8
#define RIGHTSIDE 11
//these are rows, counting from the top
//BOTTOM must be <= (LCD_WIDTH-6) and >= (TOP+4)
#define TOP 0
#define BOTTOM 64
#endif

#define NO_CALC_DETECT
#include <tigcclib.h>
#undef __need_in_use_bit
#define __need_in_use_bit ((void)0)
#include "extgraph.h"

#ifdef BINARYSCORE
//sprites for '0' and '1'
unsigned short binary[2] = {0b0100101010100100,0b0100110001001110};
#endif

//7 pieces, plus an extra T piece
//4 rotated copies of each piece
unsigned short pieces[8][4]=
{ 
	{0b0000000011110000,
	 0b0100010001000100,
	 0b0000111100000000,
	 0b0010001000100010},
	{0b0000001011100000,
	 0b0100010001100000,
	 0b0000011101000000,
	 0b0000011000100010},
	{0b0000111000100000,
	 0b0010001001100000,
	 0b0000010001110000,
	 0b0000011001000100},
	{0b0000010011100000,
	 0b0100011001000000,
	 0b0000011100100000,
	 0b0000001001100010},
	{0b0000011100100000,
	 0b0000001001100010,
	 0b0000010011100000,
	 0b0100011001000000},
	{0b0000110001100000,
	 0b0010011001000000,
	 0b0000011000110000,
	 0b0000001001100100},
	{0b0000011011000000,
	 0b0100011000100000,
	 0b0000001101100000,
	 0b0000010001100010},
	{0b0000011001100000,
	 0b0000011001100000,
	 0b0000011001100000,
	 0b0000011001100000}
};

//piece number and piece rotation
register unsigned short pn asm("d6"),pr asm("d7");
//piece x and y position
register short ppx asm("a4"),ppy asm("a3");

#ifdef HIGHSCORE
unsigned short highscore=0;
#endif

//XOR a screen pixel
void XORPix(unsigned short x,unsigned short y) {
	EXT_XORPIX(LCD_MEM,x,y);
}

//XOR a 4x4 sprite onto the display
void XORSprite(unsigned short x,unsigned short y,unsigned short sprite) {
	short sx,sy;
	for(sx=4;sx--;)
		for(sy=4;sy--;)
			if(((sprite<<(sy*4))<<sx)&(1<<15)) XORPix(x+sx,y+sy);
}

//check if the current piece collides with something on the screen
int collide() {
	short sx,sy;
	for(sx=4;sx--;)
		for(sy=4;sy--;)
			if(
				(((pieces[pn][pr]<<(sy*4))<<sx)&(1<<15))
				&&
				EXT_GETPIX(LCD_MEM,ppx+sx,ppy+sy)
			)
				return 1;
	return 0;
}

//Draw (or erase) the piece at its location.
void XORPiece() {
	//draw the piece on the tiny board
	XORSprite(ppx,ppy,pieces[pn][pr]);
	#ifdef NOTSOTINY
	short sx,sy;
#ifdef GHOSTDROP
	//calculate and maybe draw the position the piece will be in if you hit F1
	short dropy=ppy;
	short redraw=0;
	if(collide()) {
		XORSprite(ppx,ppy,pieces[pn][pr]);
		redraw=1;
	}
		while(!collide()&&ppy<BOTTOM)
			ppy++;
		ppy--;
#ifdef PERSISTENTGHOST
		short temp=ppy;
		ppy=dropy;
		XORSprite(ppx,ppy,pieces[pn][pr]);
		ppy=temp;
		if(!collide()&&ppy<BOTTOM)
#else
		if(ppy>dropy+3&&!collide()&&ppy<BOTTOM)
#endif
			for(sx=4;sx--;)
				for(sy=4;sy--;)
					if(((pieces[pn][pr]<<(sy*4))<<sx)&(1<<15)) XORSprite(
					(ppx+sx)*4,
					((ppy-TOP)+sy)*4,
					0b1101100000011011);
		ppy=dropy;
#ifdef PERSISTENTGHOST
		XORSprite(ppx,ppy,pieces[pn][pr]);
#endif
	if(redraw) {
		XORSprite(ppx,ppy,pieces[pn][pr]);
	}

#endif
	//draw the piece on the big board
	for(sx=4;sx--;)
		for(sy=4;sy--;)
			if(((pieces[pn][pr]<<(sy*4))<<sx)&(1<<15)) XORSprite(
			(ppx+sx)*4,
			((ppy-TOP)+sy)*4,
			0b1111110110111111);
#endif
}

// Main Function
void _main(void)
{
	register short timer asm("d4");
	short oldOSInitKeyInitDelay=OSInitKeyInitDelay(96);
	short oldOSInitBetweenKeyDelay=OSInitBetweenKeyDelay(14);
	short delay=2000;
	
	short x,y;
	short opr,npn;
	for(x=960;x--;)
		*(((long *)LCD_MEM)+x)=0;
	
	for(x=LEFTSIDE*8+1;x<RIGHTSIDE*8+7;x++) {
#ifdef NOTSOTINY
//#ifdef PRETTYBORDER
		XORSprite(x*4,(BOTTOM-TOP)*4,0b1111101001011010);
//#else
//		XORPix(x*4+3,(BOTTOM-TOP)*4);
//#endif
#endif
		XORPix(x,BOTTOM);
	}
	for(y=TOP;y<=BOTTOM;y++) {
#ifdef NOTSOTINY
#ifdef PRETTYBORDER
		XORSprite(0,(y-TOP)*4,0b0101101101011011);
		XORSprite(((RIGHTSIDE-LEFTSIDE+1)*8-1)*4,(y-TOP)*4,0b1010110110101101);
#else
		XORPix((LEFTSIDE*32)+3,(y-TOP)*4);
		XORPix(((RIGHTSIDE+1)*8-1)*4,(y-TOP)*4);
#endif
#endif
		XORPix(LEFTSIDE*8,y);
		XORPix(RIGHTSIDE*8+7,y);
	}
	
	while(1){
		pn=npn&7;
		npn=timer+ppy+ppx; //pseudo-psuedo-random number
		pr=0;
		ppx=(LEFTSIDE*8)+((RIGHTSIDE*8+7)-(LEFTSIDE*8))/2-1;
		ppy=TOP;
		if(collide()) break;

#if defined(BINARYSCORE)||defined(NEXTPIECE)||defined(LABEL)||defined(SPARR)
		for(x=121;--x;)
			*(((long *)LCD_MEM+750-x))=0;
#endif
#if defined(SPARR)&&defined(LABEL)
		DrawStr(80,LCD_HEIGHT-14,"By Sparr",A_NORMAL);
		DrawStr(0,LCD_HEIGHT-14,"VTTetris",A_NORMAL);
#else
#ifdef SPARR
//display author's name in bottom left
		*(long*)(LCD_MEM+(LCD_HEIGHT-5)*30)=0b11001010000110110001001100110000;
		*(long*)(LCD_MEM+(LCD_HEIGHT-4)*30)=0b10101010101000101010101010101000;
		*(long*)(LCD_MEM+(LCD_HEIGHT-3)*30)=0b11000100000100110011101100110000;
		*(long*)(LCD_MEM+(LCD_HEIGHT-2)*30)=0b10100100100010100010101010101000;
		*(long*)(LCD_MEM+(LCD_HEIGHT-1)*30)=0b11000100001100100010101010101000;
#endif
#ifdef LABEL
//display "VTTETRIS" in the bottom middle
		*(long*)(LCD_MEM+(LCD_HEIGHT-5)*30+6)=0b00010101110111011101110110010011;
		*(long*)(LCD_MEM+(LCD_HEIGHT-4)*30+6)=0b00010100100010010000100101010100;
		*(long*)(LCD_MEM+(LCD_HEIGHT-3)*30+6)=0b00010100100010011000100110010010;
		*(long*)(LCD_MEM+(LCD_HEIGHT-2)*30+6)=0b00010100100010010000100101010001;
		*(long*)(LCD_MEM+(LCD_HEIGHT-1)*30+6)=0b00001000100010011100100101010110;
#endif
#endif
#ifdef BINARYSCORE
//display your score, in binary, in the bottom right
		for(x=12;x--;)
			XORSprite(134-x*4,(LCD_HEIGHT-4),binary[((2000-delay)>>x)&1]);
#endif
#ifdef NEXTPIECE
//display the next piece on the bottom
#ifdef NOTSOTINY
	short sx,sy;
	for(sx=4;sx--;)
		for(sy=4;sy--;)
			if(((pieces[npn&7][0]<<(sy*4))<<sx)&(1<<15)) XORSprite(
			(14+sx)*4,
			(22+sy)*4,
			0b1111110110111111);
#else
		XORSprite(40,(LCD_HEIGHT-4),pieces[npn&7][0]);
#endif
#endif

		while(1){
#ifdef NOTSOTINY
			timer=delay*2;
#else
			timer=delay;
#endif
			XORPiece();
			while(timer>0) {
				timer--;
				if(kbhit()) {
					timer-=71;
					XORPiece();
					unsigned short key=ngetchx();
					if(key==KEY_RIGHT) {
						ppx++;
					  ppx-=collide();
					}
					if(key==KEY_LEFT) {
						ppx--;
						ppx+=collide();
					}
					if(key==KEY_UP) {
					  opr=pr++;
					  pr%=4;
					  ppx-=collide();
					  ppx+=collide()*2;
			  		if(collide()) {
			  			ppx--;
			  			pr=opr;
			  		}
					}
					if(key==KEY_DOWN)
					  goto drop;
#ifdef IMMDROP
					if(key==KEY_F1) {
						do {
							ppy++;
						} while(!collide());
						ppy--;
						goto drop;
					}
#endif
					if(key==KEY_ESC)
					  goto quit;
					XORPiece();
				}
			}
			XORPiece();
			drop:
			ppy++;
			if(collide()) {
				ppy--;
				break;
			}
		}
		XORPiece();
		for(y=(ppy+3)>BOTTOM-1?BOTTOM-1:(ppy+3);y>=ppy;y--) {
			for(x=LEFTSIDE;x<=RIGHTSIDE;x++)
				if(*((char*)(LCD_MEM+y*30+x))!=~0) break;
			if(x==RIGHTSIDE+1) {
				memmove(LCD_MEM+TOP*30+30,LCD_MEM+TOP*30,(y-TOP)*30);
#ifdef NOTSOTINY
				memmove(LCD_MEM+4*30,LCD_MEM,(y-TOP)*4*30);
#endif
				y++;
				delay-=(RIGHTSIDE-LEFTSIDE)*8+6;
			}
		}
		delay--;
	}
	
	quit:

	OSInitKeyInitDelay(oldOSInitKeyInitDelay);
	OSInitBetweenKeyDelay(oldOSInitBetweenKeyDelay);
}
